<?php
namespace FattorinoIt\Spedizione\Model\Carrier;

use Magento\Quote\Model\Quote\Address\RateRequest;
use Magento\Shipping\Model\Rate\Result;

use Magento\Framework\Stdlib\CookieManagerInterface;
use Magento\Framework\Stdlib\Cookie\CookieMetadataFactory;
use Magento\Framework\Session\SessionManagerInterface;
use FattorinoIt\Spedizione\Helper\FattorinoItHelper;


class Shipping extends \Magento\Shipping\Model\Carrier\AbstractCarrier implements \Magento\Shipping\Model\Carrier\CarrierInterface
{
  /**
  * @var string
  */
  //protected $_code = 'simpleshipping';
  protected $_code = 'fattorinoit';

  /**
  * @var \Magento\Shipping\Model\Rate\ResultFactory
  */
  protected $_rateResultFactory;

  /**
  * @var \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory
  */
  protected $_rateMethodFactory;

  public $sc;
  protected $_checkoutSession;

  /**
  * CookieManager
  *
  * @var CookieManagerInterface
  */
  private $cookieManager;

  /**
  * @var CookieMetadataFactory
  */
  private $cookieMetadataFactory;

  /**
  * @var SessionManagerInterface
  */
  private $sessionManager;

  private $fattorino_helper;

  /**
  * Shipping constructor.
  *
  * @param \Magento\Framework\App\Config\ScopeConfigInterface          $scopeConfig
  * @param \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory  $rateErrorFactory
  * @param \Psr\Log\LoggerInterface                                    $logger
  * @param \Magento\Shipping\Model\Rate\ResultFactory                  $rateResultFactory
  * @param \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $rateMethodFactory
  * @param array                                                       $data
  */
  public function __construct(
    \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
    \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory $rateErrorFactory,
    \Psr\Log\LoggerInterface $logger,
    \Magento\Shipping\Model\Rate\ResultFactory $rateResultFactory,
    \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $rateMethodFactory,
    \Magento\Checkout\Model\Session $_checkoutSession,
    CookieManagerInterface $cookieManager,
    CookieMetadataFactory $cookieMetadataFactory,
    SessionManagerInterface $sessionManager,
    \FattorinoIt\Spedizione\Helper\FattorinoItHelper $fattorino_helper,
    array $data = []
  ) {
    $this->sc = $scopeConfig;
    $this->_checkoutSession = $_checkoutSession;
    $this->_rateResultFactory = $rateResultFactory;
    $this->_rateMethodFactory = $rateMethodFactory;
    $this->cookieManager = $cookieManager;
    $this->cookieMetadataFactory = $cookieMetadataFactory;
    $this->sessionManager = $sessionManager;
    $this->fattorino_helper = $fattorino_helper;
    parent::__construct($scopeConfig, $rateErrorFactory, $logger, $data);
  }

  /**
  * get allowed methods
  * @return array
  */
  public function getAllowedMethods()
  {
    return [$this->_code => $this->getConfigData('name')];
  }

  /**
  * @return float
  */
  private function getShippingPrice()
  {
    $conf = $this->sc->getValue("web/unsecure/base_url");
    $configPrice = $this->getConfigData('price');
    $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
    $shippingAddress = $this->_checkoutSession->getQuote()->getShippingAddress();
    $customer = $this->_checkoutSession->getQuote()->getCustomer();

    $address = $this->fattorino_helper->returnShippingAddressAssemble($shippingAddress);
    $configPrice = -1;
    if (isset($address['postcode'])) {
      $data = array(
        'shop'=>array('domain' => preg_replace("(^https?://)", "", $conf )),
        'address' => $address
      );

      if ($this->cookieManager->getCookie('fitship')) {
        $fitship = $this->cookieManager->getCookie('fitship');
      } else {
        $fitship = date('Y-m-d H:i', strtotime(date('Y-m-d H:i')." + 4 hours"));
        $metadata = $this->cookieMetadataFactory
            ->createPublicCookieMetadata()
            ->setDuration(86400)
            ->setPath('/');


        $this->cookieManager->setPublicCookie(
            'fitship',
            $fitship,
            $metadata
        );
      }




      $rates_api = $this->fattorino_helper->getRates($data['shop'],$data['address'], $fitship);
      $configPrice = $rates_api;
      $shippingPrice = $this->getFinalPriceWithHandlingFee($configPrice);
      return $shippingPrice;
    }
    return false;
  }

  /**
  * @param RateRequest $request
  * @return bool|Result
  */
  public function collectRates(RateRequest $request)
  {
    if (!$this->getConfigFlag('active')) {
      return false;
    }

    /** @var \Magento\Shipping\Model\Rate\Result $result */
    $result = $this->_rateResultFactory->create();

    /** @var \Magento\Quote\Model\Quote\Address\RateResult\Method $method */
    $method = $this->_rateMethodFactory->create();

    $method->setCarrier($this->_code);
    $method->setCarrierTitle($this->getConfigData('title'));

    $method->setMethod($this->_code);
    $method->setMethodTitle($this->getConfigData('name'));

    $amount = $this->getShippingPrice();

    $method->setPrice($amount);
    $method->setCost($amount);

    $result->append($method);
    if ($amount == 0){
      return false;
    }
    return $result;
  }
}
