<?php
namespace FattorinoIt\Spedizione\Observer;

use Magento\Framework\Event\Observer as EventObserver;
use Magento\Framework\Event\ObserverInterface;
use Magento\Quote\Model\QuoteRepository;
use FattorinoIt\Spedizione\Model\Validator;
use FattorinoIt\Spedizione\Helper\FattorinoItHelper;
use Magento\Framework\Stdlib\CookieManagerInterface;
use Magento\Framework\Stdlib\Cookie\CookieMetadataFactory;

class SalesModelServiceQuoteSubmitBefore implements ObserverInterface
{
  /**
  * @var QuoteRepository
  */
  private $quoteRepository;

  /**
  * @var Validator
  */
  private $validator;


  private $fattorino_helper;
  private $sc;
  private $cookieManager;


  /**
  * SalesModelServiceQuoteSubmitBefore constructor.
  *
  * @param QuoteRepository $quoteRepository
  * @param Validator $validator
  */
  public function __construct(
    QuoteRepository $quoteRepository,
    \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
    CookieManagerInterface $cookieManager,
    \FattorinoIt\Spedizione\Helper\FattorinoItHelper $fattorino_helper,

    Validator $validator
  ) {
    $this->quoteRepository = $quoteRepository;
    $this->validator = $validator;
    $this->fattorino_helper = $fattorino_helper;
    $this->sc = $scopeConfig;
    $this->cookieManager = $cookieManager;


  }

  /**
  * @param EventObserver $observer
  * @return $this
  * @throws \Exception
  */
  public function execute(EventObserver $observer)
  {
    $order = $observer->getOrder();
    /** @var \Magento\Quote\Model\Quote $quote */
    $quote = $this->quoteRepository->get($order->getQuoteId());
    if (!$this->validator->validate($quote->getDeliveryDate())) {
      throw new \Exception(__('Invalid Delivery Date '.$quote->getDeliveryDate()));
    }
    $order->setDeliveryDate($quote->getDeliveryDate());
    $order->setDeliveryComment($quote->getDeliveryComment());

    $address = $this->fattorino_helper->returnShippingAddressAssemble($order->getShippingAddress());
    $conf = $this->sc->getValue("web/unsecure/base_url");


    $fields = json_encode(array(
      "order_id"=> $order->getIncrementId(),
      "customer_email"=> $order->getCustomerEmail(),
      "customer_username"=> $address['lastname']." ".$address['firstname'],
      "customer_phone"=> $address['telephone'],
      "customer_address"=> $address['street'][0] ." - ". $address['postcode']." ".$address['city'],
      "fitship"=> $this->cookieManager->getCookie('fitship'),
      "shop" => array(
        "id"=> $order->getStoreId(),
        "name"=> $order->getStoreName() ? $order->getStoreName() : preg_replace("(^https?://)", "", $conf),
        "domain"=> preg_replace("(^https?://)", "", $conf ),
        "domain_ssl"=> '',
        "url"=> $conf
      )
    ));


    $return = $this->fattorino_helper->callFattorino($fields, 'create_task_delivery');
    $return_json = json_decode($return);

    return $this;
  }
}
